package editor;

import utils.DataUsers;
import utils.Global;

/**
 * Builds a dialogue with a single text field for a new document's name. The field performs
 * a real-time check for already existing names.
 * @author hqcasanova
 * 
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this file,
 * You can obtain one at http://mozilla.org/MPL/2.0/.
 */
public class NameDoc extends FieldConfirm {
    private String sIdUsr;	//current user of the editor
    	
    /**
     * Customised constructor. Sets up the dialogue with the text field and the corresponding
     * format validation expression.
     * @param sIdUsr Current user id
     */
	public NameDoc (String sIdUsr) {
		super(Global.LANG.getString ("nombraTitulo"), Global.LANG.getString ("nombraCampoTexto"), Global.DOCNAME_REGEX);	
		this.sIdUsr = sIdUsr;		
	}
		
	/**
	 * Sets the criteria to consider a document name as correct, one of them being that the name did not
	 * exist previously.
	 * @param iIdField Text field's id that holds the document name.
	 */
	protected boolean isCorrect (int iIdField) {
		return (super.isCorrect(iIdField) && !DataUsers.existDocumentTitle (sIdUsr, getFields()[iIdField].trim()));		
	}

}
