package editor;

import utils.DataUsers;
import utils.Global;

/**
 * Builds a dialogue with confirmation for the naming and labeling of a new document.
 * @author hquintero
 * 
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this file,
 * You can obtain one at http://mozilla.org/MPL/2.0/.
 */
public class NameDocPicto extends FieldPictoConfirm {
    private String sIdUsr;	//Current user's id
    	
    /**
     * Customised constructor. Sets up a dialogue with a text field and a selectable list of pictograms.
     * @param sIdUsr Current user's id for database queries.
     */
	public NameDocPicto (String sIdUsr) {
		super (Global.LANG.getString ("nombraTitulo"), 
			   Global.LANG.getString ("nombraCampoTexto"),
			   Global.LANG.getString ("nombraListaTexto"),
			   Global.DOCNAME_REGEX);	
		this.sIdUsr = sIdUsr;
	}
		
	/**
	 * Sets the criteria to consider the text field's content as correct. Namely, that it's format
	 * is correct and that the name of the document does not exist already.
	 * @param iIdCampo Text field's id holding the new document's name.
	 */
	protected boolean isCorrect (int iIdCampo) {
		return (super.isCorrect(iIdCampo) && !DataUsers.existDocumentTitle (sIdUsr, getFields()[iIdCampo].trim()));		
	}

}

