package rtf;


import java.awt.Color;
import java.io.FileOutputStream;
import java.io.IOException;

import org.jsoup.Jsoup;
import org.jsoup.nodes.Node;
import org.jsoup.select.Elements;

import com.lowagie.text.BadElementException;
import com.lowagie.text.Cell;
import com.lowagie.text.Chunk;
import com.lowagie.text.Font;
import com.lowagie.text.FontFactory;
import com.lowagie.text.Image;
import com.lowagie.text.List;
import com.lowagie.text.ListItem;
import com.lowagie.text.Paragraph;
import com.lowagie.text.Table;
import com.lowagie.text.rtf.RtfWriter2;

/**
 * 
 * @autor mtordesillas
 * 
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this file,
 * You can obtain one at http://mozilla.org/MPL/2.0/.
 */
public class HTML2RTF extends HTMLDictionary {
	
	//Constants
	protected static String DEFAULT_FONT = "Sarakanda";
	protected static int DEFAULT_SIZE = 12;
	protected static Color DEFAULT_COLOR = Color.BLACK;
	
	//Fields
	private com.lowagie.text.Document rtfDocument;
	private org.jsoup.nodes.Document htmlDocument;
	private String defaultFontName;
	private int defaultFontSize;
	private Color defaultFontColor;
		
	//Constructor with default font parameters
	public HTML2RTF(String defaultFontName, int defaultFontSize, Color defaultFontColor){
		//Set up HTML entity dictionary
		super();
		
		//Set default font
		this.defaultFontSize = defaultFontSize;
		this.defaultFontName = defaultFontName;
		this.defaultFontColor = defaultFontColor;
		
		//Probe fonts directory and install fonts.
		FontFactory.registerDirectories();	
	}

	
	//Constructor without default font parameters
	public HTML2RTF (){
		this (DEFAULT_FONT, DEFAULT_SIZE, DEFAULT_COLOR);	
	}
	
	//Converts the HTML File into RTF Format
	public void ConvertFile(String htmlString, String destinationDocument){
		rtfDocument = new com.lowagie.text.Document(); //Creates a new RTFDocument
		try {
			htmlDocument = Jsoup.parse(htmlString, "UTF-8"); //Parses the input file
			RtfWriter2.getInstance(rtfDocument, new FileOutputStream(destinationDocument)); //Initializes the RtfWriterClass
		} catch (IOException IOE) {}
		Elements elements = htmlDocument.getElementsByTag("body"); //Gets all the body elements into a list of elements (there should be only one)
		org.jsoup.nodes.Element htmlBody = elements.first(); //Gets the first body element of the list.
		rtfDocument.open(); //Opens the RTFDocument
		org.jsoup.nodes.Element testBody;
		for(int i = 0; i < htmlBody.children().size(); i++){
			testBody = htmlBody.children().get(i);
			ParseNode(testBody);
			
		}
		rtfDocument.close();
	}
	
	
	//Parses a jsoup html Element node
	private void ParseNode(org.jsoup.nodes.Element elem){
		String tagName = elem.tagName();
		try {
		if (tagName.contentEquals("p")){
			rtfDocument.add(ParseParagraph(elem));
		} 
		else if(tagName.contentEquals("img")){
			ParseImage(elem);
		}
		else if(tagName.contentEquals("ul")){
			rtfDocument.add(ParseList(elem));
		}
		else if(tagName.contentEquals("ol")){
			rtfDocument.add(ParseList(elem));
		}
		else if(tagName.contentEquals("table")){
			rtfDocument.add(ParseTable(elem));
		}
		} catch (Exception e){
		
		}
	}
	
	//Parses an html paragraph and converts
	private Paragraph ParseParagraph(org.jsoup.nodes.Element elem){

		ChunkFormat CurrentFormat = new ChunkFormat(this.defaultFontName, this.defaultFontSize, this.defaultFontColor);
		Paragraph currentParagraph = new Paragraph();
		if(elem.children().size() >= 0){
			ParseChunk(elem, currentParagraph, CurrentFormat);
		}
		return currentParagraph;
	}

	//Parses a text chunk with a particular format.
	private void ParseChunk(org.jsoup.nodes.Element currentElement, Paragraph currentParagraph, ChunkFormat currentFormat){
		String tagName = "";
		Font f;
		ChunkFormat thisFormat = new ChunkFormat(currentFormat);
		int elementIndex = 0;
		java.util.List<Node> nodes = currentElement.childNodes();
		org.jsoup.nodes.Element elem;
		String style = currentElement.attr("style");
		String styles[] = style.split("; ");
		for (int i = 0; i < styles.length; i++){
			String styleComp[] = styles[i].split(": ");
			if (styleComp[0].equals("margin-left")){
				float margin = Float.valueOf(styleComp[1].trim()).floatValue();
				currentParagraph.setIndentationLeft(margin);
			} else if (styleComp[0].equals("margin-top")){
				float margin = Float.valueOf(styleComp[1].trim()).floatValue();
				currentParagraph.setSpacingBefore(margin);
			}
		}
		ChunkFormat parentFormat = new ChunkFormat(thisFormat);
		for (int j = 0; j < nodes.size(); j++){
			//Get the format of the current tag for the chunk
			ChunkFormat tempFormat = new ChunkFormat(currentFormat);
			if (nodes.get(j).getClass() == org.jsoup.nodes.Element.class){
				elem = currentElement.child(elementIndex);
				tagName = elem.tagName();
				if(tagName == "img"){
				currentParagraph.add(ParseImage(elem));
				}
				else {
					if(tagName == "font"){
						String font = elem.attributes().get("face");
							if(font != ""){
								tempFormat.SetChunkFont(font);
							}
						String size = elem.attributes().get("size");
							if(size != ""){
								tempFormat.SetChunkFontSize(Integer.valueOf(size));
							}
						String color = elem.attributes().get("color");
							if(color != ""){
								tempFormat.SetChunkFontColor(color);
							}
					} 
					else if(tagName == "b"){
						tempFormat.SetChunkBold(true);
					}
					else if(tagName == "i"){
						tempFormat.SetChunkItalic(true);
						//tempFormat.SetChunkBold(true);
					}
					else if(tagName == "u"){
						tempFormat.SetChunkUnderlined(true);
					}
					//Check if the chunk is a leaf of the tree
					if(elem.children().size() == 0){
					//If So, create it and add it to the paragraph
						Font tempfont = getFont(tempFormat);	
						Chunk chunk;
						//Store the element in String format
						String chunkText = elem.toString();
						//Trim the tags -- Needed instead of elem.text() to avoid the method to trim the starting space
						//chunkText = chunkText.substring(chunkText.indexOf('>', 0) + 1, chunkText.indexOf('<',1));
						chunkText = elem.text();
						chunkText = cleanHTMLCode(chunkText);
						chunk = new Chunk(chunkText);
						chunk.setFont(tempfont);
						currentParagraph.add(chunk);
						}
					
					else {
						//If not, call the function recursively
						ChunkFormat myFormat = new ChunkFormat(tempFormat);
						//ChunkFormat tempFormat = parentFormat;
						if(elem.children().size() > 0){
							ParseChunk(elem, currentParagraph, myFormat);
						}
					}
				}
				elementIndex++;
			}
			else {
				f = getFont(parentFormat);	
				String chunkText = nodes.get(j).toString();
				chunkText = cleanHTMLCode(chunkText);
				Chunk chunk = new Chunk(chunkText);
				chunk.setFont(f);
				currentParagraph.add(chunk);
			}
		}
	}
	
	private Font getFont(ChunkFormat CurrentFormat){
		Font tempfont = FontFactory.getFont(CurrentFormat.GetFontName(),CurrentFormat.GetFontSize(), FontStyle(CurrentFormat), CurrentFormat.GetColor());	
		int style = 0;
		if (tempfont.getBaseFont() == null){
			tempfont = FontFactory.getFont(CurrentFormat.GetFontName(),CurrentFormat.GetFontSize(), style, CurrentFormat.GetColor());	
		}
		if (CurrentFormat.IsChunkItalic()){
			style = style + Font.ITALIC;
		}
		if (CurrentFormat.IsChunkBold()){
			style = style + Font.BOLD;
		}
		if (CurrentFormat.IsChunkUnderlined()){
			style = style + Font.UNDERLINE;
		}
		tempfont.setStyle(style);
		return tempfont;
	}
	
	private int FontStyle(ChunkFormat format){
		int fontStyle = 0;
		if(format.IsChunkBold()){
			fontStyle = fontStyle | Font.BOLD;
		}
		if(format.IsChunkItalic()){
			fontStyle = fontStyle | Font.ITALIC;
		}
		if(format.IsChunkUnderlined()){
			fontStyle = fontStyle | Font.UNDERLINE;
		}
		return fontStyle;
	}
	
	//Converts a html image reference to a rtf image
	public Image ParseImage(org.jsoup.nodes.Element elem){
		String substring = elem.attr("src");
		try {
		Image im = Image.getInstance(substring);
			return im;
		} catch(IOException e){
			return null;
		}
		catch(BadElementException e){
			return null;
		}
		
	}
	//Parses a list structure
	private List ParseList(org.jsoup.nodes.Element elem){
		List rtfList = new List();
		if (elem.tagName() == "ol"){
			rtfList.setNumbered(true);
		}
		for(int i = 0; i < elem.children().size(); i++){
			rtfList.add(ParseListItem(elem.child(i)));
		}
		rtfList.add(new ListItem(""));
		return rtfList;
	}
	//parses a list item structure
	private ListItem ParseListItem(org.jsoup.nodes.Element elem){
		ListItem rtfListEntry = new ListItem();
		Font currentFont = FontFactory.getFont(this.defaultFontName, this.defaultFontSize, 0, this.defaultFontColor);
		rtfListEntry.setFont(currentFont);
		for(int i = 0; i < elem.children().size(); i++){
		org.jsoup.nodes.Element subElement = elem.child(i);
		String tagName = subElement.tagName();
		if (tagName.contentEquals("p")){
			rtfListEntry.add(ParseParagraph(subElement));
		} 
		else if(tagName.contentEquals("img")){
			rtfListEntry.add(ParseImage(subElement));
		}
		else if(tagName.contentEquals("table")){	
			//rtfListEntry.add(ParseTable(subElement));
		} 
		}
		return rtfListEntry;
	}
	
	private Table ParseTable(org.jsoup.nodes.Element elem){
		try{
		
		org.jsoup.nodes.Element tableBody = elem.child(0);
		int tableRows = tableBody.children().size();
		org.jsoup.nodes.Element tableRow = tableBody.child(0);
		int tableColumns = tableRow.children().size();
		
		Table rtfTable = new Table(tableColumns, tableRows);
		rtfTable.setPadding(0);
		rtfTable.setSpacing(0);
		for (int i = 0; i < tableRows; i++){
			tableRow = tableBody.child(i);
			for(int j = 0; j < tableColumns; j++){
				rtfTable.addCell(ParseTableCell(tableRow.child(j)));
			}
		}
		return rtfTable;
		} catch(Exception e) {
			return null;
		}
	}
	
	private Cell ParseTableCell(org.jsoup.nodes.Element elem){
		Cell rtfCell = new Cell();
		String cellWidth = elem.attr("width");
		int width = Integer.valueOf(cellWidth);
		rtfCell.setWidth(width);
		for(int i = 0; i < elem.children().size(); i++){
			org.jsoup.nodes.Element subElement = elem.child(i);
			String tagName = subElement.tagName();
			if (tagName.contentEquals("p")){
				rtfCell.add(ParseParagraph(subElement));
			} 
			else if(tagName.contentEquals("img")){
				rtfCell.add(ParseImage(subElement));
			}
			else if(tagName.contentEquals("table")){	
				//rtfCell.add(ParseTable(subElement));
			} 
			else if(tagName.contentEquals("ul")){	
				rtfCell.add(ParseList(subElement));
			} 
			else if(tagName.contentEquals("ol")){		
				rtfCell.add(ParseList(subElement));
			}
				
			}
		
		
		return rtfCell;
	}
	
	
	
}
	

