package utils;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.zip.CRC32;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

/**
 * The following piece of source code is an extension of that posted at
 * http://www.todoexpertos.com/categorias/tecnologia-e-internet/programacion/java/respuestas/198241/comprimir-una-carpeta
 * @author jwdd
 */
public class NZipCompresser
{
	public final static int ALLFILES = 0;
	public final static int NO_TICA_DOCS = 1;

	private String m_basePath = null;
	private File m_dir = null;
	private String m_OutputFileName;

	public NZipCompresser()
	{

	}

	// This class gets directory and compress it reqursively
	// into zip file named outputFileName

	public NZipCompresser(File directory, String outputFileName)
	{
		m_dir = directory;
		m_OutputFileName = outputFileName;
	}

	public void compress(int ticafiles) throws Exception
	{
		try
		{
			FileOutputStream zipFilename = new FileOutputStream(m_OutputFileName);
			ZipOutputStream zipoutputstream = new ZipOutputStream(zipFilename);
			m_basePath = m_dir.getPath();
			CompressDir(m_dir, zipoutputstream, ticafiles);
			zipoutputstream.setMethod(ZipOutputStream.DEFLATED);
			zipoutputstream.close();
		}

		catch (Exception e)
		{
			throw new Exception("Something wrong in compresser: " + e);
		}
	}

	public void compress() throws Exception
	{
		compress(ALLFILES);
	}

	public void setDirectory(File dir)
	{
		m_dir = dir;
	}

	public void setOutputFileName(String FileName)
	{
		m_OutputFileName = FileName;
	}

	public File getDirectory()
	{
		return m_dir;
	}

	public String getOutputFileName()
	{
		return m_OutputFileName;
	}

	// Walker through directory structure

	private void CompressDir(File f, ZipOutputStream zipoutputstream, int ticafiles)
	{
		if(f.isDirectory())
		{
			File[] files = f.listFiles();
			int i;
			for (i = 0; i < files.length; i++)
			{
				if(files[i].isDirectory())
				{
					char c = files[i].getName().charAt(0);
					String pc = c + " ";
					pc = pc.trim();
					if(!pc.equals("."))
					{
						CompressDir(files[i], zipoutputstream, ticafiles);
					}
				}
				if(ticafiles == ALLFILES)
				{
					if(files[i].isFile())
					{
						char c = files[i].getName().charAt(0);
						String pc = c + " ";
						pc = pc.trim();
						if(!pc.equals("."))
						{
							// System.out.println("Primer caracter " + pc);
							// System.out.println("adding file:" + files[i].getPath());
							addOneFile(files[i], zipoutputstream);
						}
					}
				}
				else if(ticafiles == NO_TICA_DOCS)
				{
					if(files[i].isFile() && !DataUsers.isTICaDocFile(files[i].getName()))
					{
						char c = files[i].getName().charAt(0);
						String pc = c + " ";
						pc = pc.trim();
						if(!pc.equals("."))
						{
							// System.out.println("Primer caracter " + pc);
							// System.out.println("adding file:" + files[i].getPath());
							addOneFile(files[i], zipoutputstream);
						}
					}
				}
			}
		}
	}

	// Actualy compress the file

	private void addOneFile(File file, ZipOutputStream zipoutputstream)
	{
		ZipEntry zipentry = new ZipEntry(file.getPath().substring(m_basePath.length() + 1));
		FileInputStream fileinputstream;
		CRC32 crc32 = new CRC32();

		byte[] rgb = new byte[1024];
		int n;

		// Compute CRC of input stream
		try
		{
			fileinputstream = new FileInputStream(file);
			while ((n = fileinputstream.read(rgb)) > -1)
			{
				crc32.update(rgb, 0, n);
			}
			fileinputstream.close();
		}

		catch (Exception e)
		{
			System.out.println("Error in computing CRC:");
			e.printStackTrace();
		}

		// Set Up Zip Entry
		zipentry.setSize(file.length());
		zipentry.setTime(file.lastModified());
		zipentry.setCrc(crc32.getValue());

		// Write Data
		try
		{
			zipoutputstream.putNextEntry(zipentry);
			fileinputstream = new FileInputStream(file);

			while ((n = fileinputstream.read(rgb)) > -1)
			{
				zipoutputstream.write(rgb, 0, n);
			}

			fileinputstream.close();
			zipoutputstream.closeEntry();
		}

		catch (Exception ex)
		{
			System.out.println("Error in writing data:");
			ex.printStackTrace();
		}
	}

	public static void main(String args[])
	{
		// NZipCompresser c = new NZipCompresser(new File(args[0]), args[1]);
		NZipCompresser c = new NZipCompresser(new File("/home/walther/book"), "/tmp/cosa.zip");
		try
		{
			c.compress();
		}

		catch (Exception e)
		{

			System.out.println(e.toString());
		}
	}
}
